import pandas as pd
import os
import re
import plotly.express as px
import pycountry
import numpy as np
from datetime import datetime
import warnings
import pyreadr
warnings.filterwarnings('ignore')
import matplotlib.pyplot as plt
import altair as alt
from sklearn.preprocessing import MinMaxScaler
scaler = MinMaxScaler()
from load_func import *
# Install "pip install --upgrade --pre kaleido" in case of issue for exporting the figures
import kaleido

"""# To Run by default"""

# Change the working directory
#os.chdir(r'F:\Storage\GEF\Code\Code_articleBM')

# Define paths
production_files = r'./Dataset/S&P/Production_clean'
ownership_folder = r'./Dataset/S&P/mergedup10'
coordinates_file = r'./Dataset/S&P/Coordinates/Coord.xls'
country_file = r'./Dataset/S&P/COMPANY_NAME (updated).xlsx'

# Process ownership files
processed = processing_assets(ownership_folder)

# Load and clean coordinates data
coord = pd.read_excel(coordinates_file, header=3)
coord = coord[['PROP_ID', 'COUNTRY_NAME', 'SNL_GLOBAL_REGION']]

# Load and clean country name data
country_name = pd.read_excel(country_file, sheet_name='updated')
country_name = (
    country_name[['COMPANY_NAME', 'Country']]
    .query('Country != "(Invalid Identifier)"')
    .drop_duplicates()
    .rename(columns={'COMPANY_NAME': 'Financial_name'})
)


# Create country lists

country_list = ['Oman', 'Algeria', 'Uzbekistan',  'Spain',
       'Zimbabwe', 'Portugal', 'Mauritius', 'Bermuda', 'Malta',
       'North Macedonia', 'Malawi',  'Myanmar',
       'Ethiopia', 'Botswana', 'Ukraine', 'Vietnam', "Côte d'Ivoire",
       'Singapore', 'United Arab Emirates', 'Kyrgyzstan', 'Ghana',
       'BVI', 'Malaysia', 'Namibia', 'Colombia', 'Bahamas',
       'Burkina Faso', 'Saudi Arabia', 'Eritrea', 'Laos', 'Belgium',
       'Azerbaijan', 'Hong Kong', 'South Korea', 'Norway', 'India',
       'Liberia', 'Argentina', 'Finland',  'Zambia', 'Venezuela',
        'Angola', 'Luxembourg', 'Türkiye', 'Morocco', 'Bulgaria',
       'New Zealand', 'Cyprus', 'North Korea', 'Tajikistan', 'Lesotho',
       'Philippines', 'Sudan', 'Cayman Islands', 'Taiwan',
       'Papua New Guinea', 'Tanzania', 'Georgia', 'Guinea',
       'Sierra Leone', 'Panama', 'Austria', 'Mexico', 'Senegal',
       'Mongolia', 'Greece', 'Iran', 'Suriname', 'Dominican Republic',
       'Poland', 'Egypt', 'Mauritania',
       'Sweden', 'Armenia', 'Switzerland', 'Japan', 'Mali',
        'Netherlands', 'Kazakhstan',  'Peru', 'Thailand',
       'Cuba', 'Niger', 'Mozambique', 'New Caledonia', 'Israel',
        'Bolivia', 'French Guiana',  'Ireland',
       'Romania', 'Serbia', 'Fiji', 'Bosnia & Herzegovina', 'Nicaragua',
       'Honduras', 'Pakistan', 'Ecuador', 'Guyana', 'Nigeria',
        'Cambodia','Jamaica', 'Guatemala', 'Madagascar',   'Gabon',
        'Italy',  'Germany', 'Liechtenstein', 'Jersey',
       'Czech Republic', 'Turkey',   'Denmark',
       'Lebanon',  'Hungary', 'Guernsey',
       'Isle of Man', 'Rep. Of the Congo', 'Cameroon', 'Yemen', 'Greenland', 'Uganda', 'Jordan', 'Burundi' ]

country_list_bolivia = ['Oman', 'Algeria', 'Uzbekistan',  'Spain',
       'Zimbabwe', 'Portugal', 'Mauritius', 'Bermuda', 'Malta',
       'North Macedonia', 'Malawi',  'Myanmar',
       'Ethiopia', 'Botswana', 'Ukraine', 'Vietnam', "Côte d'Ivoire",
       'Singapore', 'United Arab Emirates', 'Kyrgyzstan', 'Ghana',
       'BVI', 'Malaysia', 'Namibia', 'Colombia', 'Bahamas',
       'Burkina Faso', 'Saudi Arabia', 'Eritrea', 'Laos', 'Belgium',
       'Azerbaijan', 'Hong Kong', 'South Korea', 'Norway', 'India',
       'Liberia', 'Argentina', 'Finland',  'Zambia', 'Venezuela',
        'Angola', 'Luxembourg', 'Türkiye', 'Morocco', 'Bulgaria',
       'New Zealand', 'Cyprus', 'North Korea', 'Tajikistan', 'Lesotho',
       'Philippines', 'Sudan', 'Cayman Islands', 'Taiwan',
       'Papua New Guinea', 'Tanzania', 'Georgia', 'Guinea',
       'Sierra Leone', 'Panama', 'Austria', 'Mexico', 'Senegal',
       'Mongolia', 'Greece', 'Iran', 'Suriname', 'Dominican Republic',
       'Poland', 'Egypt', 'Mauritania',
       'Sweden', 'Armenia', 'Switzerland', 'Japan', 'Mali',
        'Netherlands', 'Kazakhstan',  'Peru', 'Thailand',
       'Cuba', 'Niger', 'Mozambique', 'New Caledonia', 'Israel',
       'French Guiana',  'Ireland',
       'Romania', 'Serbia', 'Fiji', 'Bosnia & Herzegovina', 'Nicaragua',
       'Honduras', 'Pakistan', 'Ecuador', 'Guyana', 'Nigeria',
        'Cambodia','Jamaica', 'Guatemala', 'Madagascar',   'Gabon',
        'Italy',  'Germany', 'Liechtenstein', 'Jersey',
       'Czech Republic', 'Turkey',   'Denmark',
       'Lebanon',  'Hungary', 'Guernsey',
       'Isle of Man', 'Rep. Of the Congo', 'Cameroon' ]


# Create color maps
color_map = {
    'Indonesia': '#FF6347',  # Tomato color for Indonesia
    'Chile': '#4682B4',      # Steel Blue color for Chile
    'Dem. Rep. Congo': '#8A2BE2',  # Blue Violet color for Dem. Rep. Congo
    'USA': '#636EFA',
    'China': '#EF553B',
    'Russia': '#00CC96',
    'Australia': '#AB63FA',
    'Canada': '#FFA15A',
    'Brazil': '#19D3F3',
    'India': '#FF6692',
    'South Africa': '#FF97FF',
    'United Kingdom': '#8B008B',
    'Other countries': '#7F7F7F'  # Default color for 'Other countries'
}


color_map_alter = {
    'Indonesia': '#FF4500',  # Orange Red for Indonesia (strong and distinct)
    'Chile': '#4682B4',      # Keep Steel Blue for Chile (calm but clear)
    'Dem. Rep. Congo': '#6A5ACD',  # Slate Blue for Dem. Rep. Congo (slightly softer than Blue Violet)
    'USA': '#1E90FF',        # Dodger Blue for USA (vibrant blue, easily recognizable)
    'China': '#DC143C',      # Crimson for China (bold red with cultural relevance)
    'Russia': '#228B22',     # Forest Green for Russia (deep green, stands out without being harsh)
    'Australia': '#20B2AA',  # Light Sea Green for Australia (distinct but not too bright)
    'Canada': '#FF6347',     # Tomato Red for Canada (vivid and energetic)
    'Brazil': '#32CD32',     # Lime Green for Brazil (bright and refreshing)
    'India': '#FF8C00',      # Dark Orange for India (strong and warm)
    'South Africa': '#9932CC',# Dark Orchid for South Africa (deep purple, royal and unique)
    'United Kingdom': '#4169E1', # Royal Blue for the United Kingdom (sophisticated and recognizable)
    'Other countries': '#7F7F7F'  # Keep gray for 'Other countries'
}


color_map_mineral = {
    'Niobium': '#FF4500',  # OrangeRed
    'Vanadium': '#FFD700',  # Gold
    'Tungsten': '#808080',  # Gray
    'Lanthanides': '#483D8B',  # DarkSlateBlue
    'Platinum': '#E5E4E2',  # Platinum
    'Cobalt': '#0000FF',  # Blue
    'Palladium': '#D3D3D3',  # LightGray
    'Chromite': '#A52A2A',  # Brown
    'Lithium': '#00FF00',  # Lime
    'Bauxite': '#8B4513',  # SaddleBrown
    'Tin': '#D2B48C',  # Tan
    'Graphite': '#2F4F4F',  # DarkSlateGray
    'Uranium': '#7FFF00',  # Chartreuse
    'Manganese': '#FFA500',  # Orange
    'Iron': '#B22222',  # FireBrick
    'Lead': '#696969',  # DimGray
    'Zinc': '#9ACD32',  # YellowGreen
    'Nickel': '#4682B4',  # SteelBlue
    'Silver': '#C0C0C0',  # Silver
    'Copper': '#FF6347',  # Tomato
}



# Create various filters


mineral_filter = ["Antimony" ,"Coal", "Gold", "Diamonds", "Lanthanum" ,"Phosphate"]

# Africa
africa = ['Guinea', 'South Africa', 'Mozambique', 'Madagascar', 'Dem. Rep. Congo', 'Morocco', 'Zambia', 'Mauritania', 'Botswana', 'Namibia', 'Eritrea', 'Zimbabwe', 'Lesotho', 'Angola', 'Sierra Leone', 'Tanzania', "Côte d'Ivoire", 'Ghana', 'Sudan', 'Liberia', 'Burkina Faso', 'Senegal', 'Nigeria', 'Egypt', 'Ethiopia', 'Algeria', 'Gabon', 'Malawi', 'Niger']

# North America
north_america = ['Jamaica', 'USA', 'Canada', 'Mexico', 'Cuba', 'Dominican Republic', 'Panama', 'Honduras', 'United Kingdom', 'Nicaragua', 'Guatemala']

# South America
south_america = ['Brazil', 'Colombia', 'Peru', 'Argentina', 'Chile', 'Ecuador', 'Bolivia', 'Venezuela', 'Guyana', 'Suriname', 'French Guiana']

# Europe
europe = ['Russia', 'Poland', 'Ukraine', 'Finland', 'Spain', 'Sweden', 'Portugal', 'Romania', 'Serbia', 'North Macedonia', 'Bulgaria', 'Greece', 'Cyprus', 'United Kingdom', 'Bosnia & Herzegovina', 'Norway', 'Austria', 'Ireland']

# Asia
asia = ['Saudi Arabia', 'China', 'India', 'Türkiye', 'Kazakhstan', 'Indonesia', 'Mongolia', 'Philippines', 'Armenia', 'Oman', 'Uzbekistan', 'Kyrgyzstan', 'Azerbaijan', 'Myanmar', 'Laos', 'Pakistan', 'Iran', 'Vietnam', 'Japan', 'Georgia', 'Cambodia', 'Tajikistan', 'Malaysia', 'North Korea', 'Israel']

# Oceania
oceania = ['Australia', 'New Zealand', 'New Caledonia', 'Papua New Guinea', 'Fiji']




# main.py
with open('Module_statistics.py', 'r') as file:
    exec(file.read())

print("Module_statistics: done")

with open('Module_production.py', 'r') as file:
    exec(file.read())

print("Module_production: done")

with open('Module_reserves.py', 'r') as file:
    exec(file.read())

print("Module_reserves: done")

with open('Module_GPD.py', 'r') as file:
    exec(file.read())

print("Module_GPD_alter: done")

with open('Module_alter_measures.py', 'r') as file:
    exec(file.read())

print("Module_alter_measures: done")

with open('Module_other_figures.py', 'r') as file:
    exec(file.read())

print("Module_other_figures: done")