"""########################## Get HHIs (PRODUCTION)##########################"""

"""
## GPD World Map
"""

#Prepare GDP dataset - Uncomment if you want to run it.


result = pyreadr.read_r(r"./Dataset/Geopolitical distance/UNVotes_new.RData")
df1 = result["completeVotes"]
df1_countryname = df1[["Countryname", 'Country']]
df1_countryname = df1_countryname.drop_duplicates()

df_2022 = df1[['rcid', 'vote', 'Country' ]]


dfs = []
unique_countries = df1['Country'].unique()

for country1 in unique_countries:
    subset_c1 = df1.query('year == 2022 & Country == @country1 & vote != 8 & vote != 9')[["rcid", "vote"]]
    subset_c1 = subset_c1.rename(columns={'vote': 'vote_c1'})

    for country2 in unique_countries:
        if country1 == country2:
            continue

        subset_c2 = df1.query('year == 2022 & Country == @country2 & vote != 8 & vote != 9')[["rcid", "vote"]]
        subset_c2 = subset_c2.rename(columns={'vote': 'vote_c2'})

        subset = pd.merge(subset_c1, subset_c2, on='rcid', how='left')

        # Handling missing values in 'vote_c2' after merge
        if subset['vote_c2'].isnull().any():
            subset = subset.dropna(subset=['vote_c2'])

        if not subset.empty:
            x = geopolitical_distance_alter(subset, subset['vote_c1'], subset['vote_c2'], 2)
            df = pd.DataFrame({'Country1': [country1], 'Country2': [country2], 'GPD': [x]})
            dfs.append(df)

# Combine all DataFrames in the list into a single DataFrame
final_df = pd.concat(dfs, ignore_index=True)
final_df = final_df.drop_duplicates()

combine = pd.merge(final_df, df1_countryname, left_on='Country1',right_on='Country', how='left')
combine = pd.merge(combine, df1_countryname, left_on='Country2',right_on='Country', how='left')

combine = combine[['Country1', 'Country2', 'GPD', 'Countryname_x',
       'Countryname_y']]
combine = combine.rename(columns={'Countryname_x': 'Countryname_1', 'Countryname_y': 'Countryname_2' })
combine = combine.drop_duplicates()
combine.to_csv(r"./Dataset/Geopolitical distance/gpd_alignment_alter.csv")



# Run the analysis

gef = pd.read_csv(r"./Dataset/Geopolitical Distance/gpd_alignment_alter.csv")
gef = gef[gef['Countryname_1'] != "United States"]
gef = gef[gef['Countryname_2'] != "United States"]
gef_grouped = gef.groupby('Country1')['GPD'].mean().reset_index()
gef_grouped = gef_grouped.sort_values(by='GPD', ascending=True)


import plotly.graph_objects as go
fig = go.Figure(data=go.Choropleth(
    locations = gef_grouped['Country1'],
    z = gef_grouped['GPD'],
    text = gef_grouped['Country1'],
    colorscale = 'Blues',
    autocolorscale=False,
    reversescale=True,
    marker_line_color='darkgray',
    marker_line_width=0.5,
    colorbar_title = 'Geopolitical<br>distance (GPD)',
))

fig.update_layout(
    geo=dict(
        showframe=False,
        showcoastlines=False,
        projection_type='equirectangular'
    )
)
fig.update_layout(legend=dict(x=0.1))
fig.update_layout(legend_orientation="h")
fig.update_coloraxes(showscale=False)
fig.update_layout(margin={"r":0,"t":0,"l":0,"b":0})

fig.write_image("./Graphs/Figure4.png")




"""
## Figure 15
"""
# Load and preprocess geopolitical distance data
df_alignment = pd.read_csv(r"./Dataset/Geopolitical Distance/gpd_alignment_alter.csv")
df_alignment = df_alignment[df_alignment['Countryname_1'] != "United States"]
df_alignment = df_alignment[df_alignment['Countryname_2'] != "United States"]

#Standardize country names
df_alignment['Countryname_1'] = df_alignment['Countryname_1'].str.replace('United States','USA')
df_alignment['Countryname_2'] = df_alignment['Countryname_2'].str.replace('United States','USA')
df_alignment['Countryname_1'] = df_alignment['Countryname_1'].str.replace('USA of America','USA')
df_alignment['Countryname_2'] = df_alignment['Countryname_2'].str.replace('USA of America','USA')


df_alignment['Countryname_1'] = df_alignment['Countryname_1'].str.replace('Democratic Republic of the Congo','Dem. Rep. Congo')
df_alignment['Countryname_2'] = df_alignment['Countryname_2'].str.replace('Democratic Republic of the Congo','Dem. Rep. Congo')

# Rename columns and normalize GPD values
df_alignment = df_alignment.rename(columns={'Countryname_1': 'COUNTRY_NAME'})
df_alignment["GPD"] = df_alignment["GPD"].replace([-1], 0)
df_alignment['normalized_GPD'] = scaler.fit_transform(df_alignment[['GPD']])
df_alignment = df_alignment.rename(columns={'COUNTRY_NAME': 'Countryname_origin'})
df_alignment = df_alignment.rename(columns={'Countryname_2': 'COUNTRY_NAME'})

# Aggregate by country
df_alignment = df_alignment.groupby(["COUNTRY_NAME"]).agg({'GPD':'mean','normalized_GPD':'mean'}).reset_index()



# Process geographic HHI data
HHI_agregated_geo = hhi_geo_df.loc[hhi_geo_df['COUNTRY_NAME'] != "All"]
HHI_agregated_geo = HHI_agregated_geo.loc[HHI_agregated_geo['HHI_reserves'] > 0]
HHI_agregated_geo = HHI_agregated_geo.groupby(["Mineral", "COUNTRY_NAME"]).agg({'HHI_reserves':'sum','value':'sum'}).reset_index()
HHI_agregated_geo = HHI_agregated_geo.sort_values(by=['HHI_reserves'], ascending=True)
HHI_agregated_geo['normalized_HHI'] = scaler.fit_transform(HHI_agregated_geo[['HHI_reserves']])


# Merge and aggregate geographic data
df_agregated_geo = pd.merge(HHI_agregated_geo,df_alignment, left_on='COUNTRY_NAME', right_on='COUNTRY_NAME' )
df_agregated_geo = df_agregated_geo.drop_duplicates()
df_agregated_geo = df_agregated_geo.groupby(['Mineral']).agg({'HHI_reserves':'sum','normalized_GPD':'mean'}).reset_index()

# Process legal origin HHI data
HHI_agregated_lo = hhi_lo_df.loc[hhi_lo_df['OWNER_COUNTRY'] != "All"]
HHI_agregated_lo = HHI_agregated_lo.loc[HHI_agregated_lo['HHI_reserves'] > 0]
HHI_agregated_lo = HHI_agregated_lo.groupby(["Mineral", "OWNER_COUNTRY"]).agg({'HHI_reserves':'sum','value':'sum'}).reset_index()
HHI_agregated_lo = HHI_agregated_lo.sort_values(by=['HHI_reserves'], ascending=True)
HHI_agregated_lo['normalized_HHI'] = scaler.fit_transform(HHI_agregated_lo[['HHI_reserves']])


# Merge and aggregate legal origin data
df_agregated_lo = pd.merge(HHI_agregated_lo,df_alignment, left_on='OWNER_COUNTRY', right_on='COUNTRY_NAME' )
df_agregated_lo = df_agregated_lo.drop_duplicates()
df_agregated_lo = df_agregated_lo.groupby(['Mineral']).agg({'HHI_reserves':'sum','normalized_GPD':'mean'}).reset_index()

# Process soc HHI data
HHI_agregated_soc = hhi_soc_df.loc[hhi_soc_df['HHI'] > 0]
HHI_agregated_soc = HHI_agregated_soc.groupby(["mineral", "Country"]).agg({'HHI':'sum'}).reset_index()
HHI_agregated_soc = HHI_agregated_soc.sort_values(by=['HHI'], ascending=True)
HHI_agregated_soc['normalized_HHI'] = scaler.fit_transform(HHI_agregated_soc[['HHI']])

# Merge and aggregate soc data
df_agregated_soc = pd.merge(HHI_agregated_soc,df_alignment, left_on='Country', right_on='COUNTRY_NAME' )
df_agregated_soc = df_agregated_soc.drop_duplicates()
df_agregated_soc = df_agregated_soc.groupby(["mineral"]).agg({'HHI':'sum','normalized_GPD':'mean'}).reset_index()



# Combine all data types
df_agregated_geo = df_agregated_geo.rename(columns={'HHI_reserves': 'HHI'})
df_agregated_geo['Type'] = "Geographic"
df_agregated_lo = df_agregated_lo.rename(columns={'HHI_reserves': 'HHI'})
df_agregated_lo['Type'] = "Legal origin"
df_agregated_soc = df_agregated_soc.rename(columns={'mineral': 'Mineral'})
df_agregated_soc['Type'] = "SOC"


df_agg_combine = pd.concat([df_agregated_geo, df_agregated_lo, df_agregated_soc], axis=0)
df_agg_combine = df_agg_combine.sort_values(by=['Mineral'], ascending=True)


# Update type names
df_agg_combine['Type'] = df_agg_combine['Type'].replace({'Legal origin': 'Direct ownership', 'SOC': 'Equity ownership'})

# Define a color mapping for categorical data in the "Type" column
categories = df_agg_combine["Type"].unique()  # Unique categories in the "Type" column
color_map = {category: color for category, color in zip(categories, px.colors.qualitative.Plotly)}

#Add small jitter to Lanthanides so that bubbles do not perfectly overlap
df_agg_combine.loc[(df_agg_combine['Mineral'] == 'Lanthanides') & (df_agg_combine['Type'] == 'Direct ownership'), 'normalized_GPD'] += 0.01

df_agg_combine = df_agg_combine.sort_values(by='Mineral', ascending=False)

# Create a new figure
fig = go.Figure()

# Loop through each category and create a separate trace
for category in categories:
    category_data = df_agg_combine[df_agg_combine["Type"] == category]
    fig.add_trace(go.Scatter(
        x=category_data["normalized_GPD"],
        y=category_data["Mineral"],
        mode='markers',
        marker=dict(
            size=category_data["HHI"]/10,  # Size based on HHI
            sizemin=3,  # Minimum size for markers
            sizemode='area',
            color=color_map[category],  # Assign color based on category
            line=dict(width=1, color='DarkSlateGrey')  # Optional border around markers
        ),
        name=category  # Category name for the legend
    ))

# Update layout for titles and theme
fig.update_layout(
    #title='Scatter Plot of Minerals by Normalized GPD and HHI',
    xaxis_title='NORMALIZED GPD',
    #yaxis_title='Mineral',
    legend_title_text='Type',
    template='plotly_white',  # Use a cleaner theme
    width=1200,  # Adjust the width
    height=800,  # Adjust the height
)

fig.update_layout(
    legend=dict(
        x=0.85,
        y=0.95,
        traceorder="reversed",
        title_font_family="Times New Roman",
        font=dict(
            family="Courier",
            size=12,
            color="black"
        ),
        bgcolor="LightSteelBlue",
        bordercolor="Black",
        borderwidth=2
    )
)


fig.write_image("./Graphs/Figure15.png")
