
"""########################## Figures 14a and 14b ##########################"""

# Define a function to process files and extract relevant data
def process_files(file_path, result_path):
    dfs = []
    for filename in os.listdir(file_path):
        try:
            match = re.search(r'([A-Za-z]+)(?:_LT|\.xlsx)', filename)
            if match:
                mineral = match.group(1)
                df = pd.read_excel(f'{result_path}/{filename}')
                df['Mineral'] = mineral
                dfs.append(df)
                print(mineral)
        except Exception as e:
            print(f"Error processing {filename}: {e}")
    return pd.concat(dfs, ignore_index=True)

# Process geographic concentration data for ongoing and project mines
geo_ongoing_path = './Dataset/S&P/HHI_reserves_geo_ongoing'
geo_project_path = './Dataset/S&P/HHI_reserves_geo_project'

ongoing_df = process_files('./Dataset/S&P/Reserves_clean', geo_ongoing_path)
project_df = process_files('./Dataset/S&P/Reserves_clean', geo_project_path)

ongoing_df = ongoing_df[(ongoing_df['COUNTRY_NAME'] == "All") & (ongoing_df['HHI_reserves'] > 0)]
project_df = project_df[(project_df['COUNTRY_NAME'] == "All") & (project_df['HHI_reserves'] > 0)]

# Prepare data for plotting
ongoing_df = ongoing_df.rename(columns={'HHI_reserves': 'HHI_reserves_past'})[['Mineral', 'HHI_reserves_past']]
project_df = project_df.rename(columns={'HHI_reserves': 'HHI_reserves_future'})[['Mineral', 'HHI_reserves_future']]

ongoing_df['HHI_reserves_past'] /= 10000
project_df['HHI_reserves_future'] /= 10000

combined_df = pd.merge(ongoing_df, project_df, how='left', on='Mineral')

# Create Altair scatter plot for geographic concentration
rect_data = pd.DataFrame({"x1": [0], "x2": [0.25], "y1": [0], "y2": [0.25]})

geo_graph = alt.Chart(combined_df).mark_point().encode(
    x=alt.X('HHI_reserves_past', scale=alt.Scale(domain=[0, 1]), title='Geographic concentration (Ongoing mines)'),
    y=alt.Y('HHI_reserves_future', scale=alt.Scale(domain=[0, 1]), title='Geographic concentration (projects)'),
    color=alt.Color('Mineral', legend=None),
).properties(width=800, height=800)

rect_geo = alt.Chart(rect_data).mark_rect(opacity=0.3).encode(
    x="x1",
    x2="x2",
    color=alt.ColorValue("#61faab")
)

rect2_geo = alt.Chart(rect_data).mark_rect(opacity=0.3).encode(
    y="y1",
    y2="y2",
    color=alt.ColorValue("#4016fa")
)

text_geo = geo_graph.mark_text(
    align='left',
    baseline='middle',
    dx=7,
    dy=-7
).encode(text='Mineral')

graph1 = geo_graph + rect_geo + rect2_geo + text_geo
graph1.save('./Graphs/Figure14a.png')

# Process ownership concentration data
owner_ongoing_path = './Dataset/S&P/HHI_reserves_countryoforigin_ongoing'
owner_project_path = './Dataset/S&P/HHI_reserves_countryoforigin_project'

ongoing_owner_df = process_files('./Dataset/S&P/Reserves_clean', owner_ongoing_path)
project_owner_df = process_files('./Dataset/S&P/Reserves_clean', owner_project_path)

ongoing_owner_df = ongoing_owner_df[(ongoing_owner_df['OWNER_COUNTRY'] == "All") & (ongoing_owner_df['HHI_reserves'] > 0)]
project_owner_df = project_owner_df[(project_owner_df['OWNER_COUNTRY'] == "All") & (project_owner_df['HHI_reserves'] > 0)]

ongoing_owner_df = ongoing_owner_df.rename(columns={'HHI_reserves': 'HHI_reserves_owner_ongoing'})[['Mineral', 'HHI_reserves_owner_ongoing']]
project_owner_df = project_owner_df.rename(columns={'HHI_reserves': 'HHI_reserves_owner_project'})[['Mineral', 'HHI_reserves_owner_project']]

ongoing_owner_df['HHI_reserves_owner_ongoing'] /= 10000
project_owner_df['HHI_reserves_owner_project'] /= 10000

combined_owner_df = pd.merge(ongoing_owner_df, project_owner_df, how='left', on='Mineral')

# Create Altair scatter plot for ownership concentration
owner_graph = alt.Chart(combined_owner_df).mark_point().encode(
    x=alt.X('HHI_reserves_owner_project', scale=alt.Scale(domain=[0, 1]), title='Ownership concentration (projects)'),
    y=alt.Y('HHI_reserves_owner_ongoing', scale=alt.Scale(domain=[0, 1]), title='Ownership concentration (Ongoing mines)'),
    color=alt.Color('Mineral', legend=None),
).properties(width=800, height=800)

rect_owner = alt.Chart(rect_data).mark_rect(opacity=0.3).encode(
    x="x1",
    x2="x2",
    color=alt.ColorValue("#61faab")
)

rect2_owner = alt.Chart(rect_data).mark_rect(opacity=0.3).encode(
    y="y1",
    y2="y2",
    color=alt.ColorValue("#4016fa")
)

text_owner = owner_graph.mark_text(
    align='left',
    baseline='middle',
    dx=7,
    dy=-7
).encode(text='Mineral')

graph2 = owner_graph + rect_owner + rect2_owner + text_owner
graph2.save('./Graphs/Figure14b.png')
