import pandas as pd
import os
import re
import plotly.express as px
import pycountry
import numpy as np
from datetime import datetime
import warnings
import pyreadr
warnings.filterwarnings('ignore')
import matplotlib.pyplot as plt
import altair as alt
from sklearn.preprocessing import MinMaxScaler
scaler = MinMaxScaler()



"""# General statistics
"""


"""# Trade restrictions
"""

#gta = pd.read_excel(r"./Dataset/GTA/GTA+Database+2021-latest.xlsx")
gta = pyreadr.read_r(r"./Dataset/GTA/GTA+Database+2008-latest.Rdata") # also works for Rds
gta = gta["master"]

trade_codes = pd.read_excel(r"./Dataset/Trade codes/Trade_codes_HS.xlsx")
trade_codes['HS2007'] = trade_codes['HS2007'].astype(object)

gta['Affected Products'] = gta['Affected Products'].astype(str)
trade_codes['HS2007'] = trade_codes['HS2007'].astype(str)

# Function to check if any of the comma-separated IDs are in codes
def id_in_codes(id_str, codes_set):
    # Split the comma-separated IDs and strip whitespace
    id_list = [x.strip() for x in id_str.split(',')]
    # Return True if any ID is in the codes set
    return any(id in codes_set for id in id_list)

# Create a set of codes for faster lookup
codes_set = set(trade_codes['HS2007'])

# Filter df1 based on the custom function
filtered_df = gta[gta['Affected Products'].apply(lambda x: id_in_codes(x, codes_set))]

# Convert the 'date' column to datetime format
filtered_df['Announcement Date'] = pd.to_datetime(filtered_df['Announcement Date'])

# Extract only the year from the 'date' column
filtered_df['year'] = filtered_df['Announcement Date'].dt.year


# List of strings to replace with 'other'
strings_to_replace = [
    'Anti-circumvention', 'Anti-dumping', 'Competitive devaluation', 'Export ban', 
    'Export licensing requirement', 'Export quota', 'Export tax', 
    'Export-related non-tariff measure, nes', 'Import ban', 'Import licensing requirement', 
    'Import quota', 'Internal taxation of imports', 'Other export incentive', 
    'Public procurement preference margin', 'State loan', 'Tax-based export incentive', 
    'Trade finance', 'Anti-subsidy', 'Capital injection and equity stakes (including bailouts)', 
    'Financial assistance in foreign market', 'Import incentive', 
    'Import tariff quota', 'Import-related non-tariff measure, nes', 'Instrument unclear', 
    'Interest payment subsidy', 'Loan guarantee', 'Local content incentive', 
    'Local labour requirement', 'Production subsidy', 'Public procurement access', 'Safeguard', 'State aid, nes', 
    'Tax or social insurance relief', 'Trade payment measure', 'Export subsidy', 
    'Local content requirement', 'Technical barrier to trade', 
    'FDI: Entry and ownership rule', 'FDI: Financial incentive', 
    'Local value added incentive', 'Import monitoring', 'In-kind grant', 
    'State aid, unspecified', 'Controls on commercial transactions and investment instruments', 
    'Public procurement, nes', 'Price stabilisation', 
    'FDI: Treatment and operations, nes', 'Local supply requirement for exports', 
    'Export tariff quota', 'Intellectual property protection', 
    'Local value added requirement'
]
filtered_df_bis = filtered_df
# Replace the specified strings with 'other'
filtered_df_bis['Intervention Type'] = filtered_df_bis['Intervention Type'].replace(strings_to_replace, 'Other types of trade barrier')

gta_year = filtered_df_bis.groupby(['year', 'Intervention Type'])['Intervention ID'].count().reset_index()
gta_year = gta_year.query('year<2024')

fig = px.bar(gta_year, x='year', y='Intervention ID', color="Intervention Type",  title="Trends in the number of public interventions considered to be barriers to international trade in critical minerals",
                          labels={
                     "Intervention ID": "Nb of intervention",
                     "year": ""})
fig.update_layout(
    legend_title='',
    template='plotly_white',  # Use a cleaner theme
    width=1200,  # Adjust the width
    height=800,  # Adjust the height
    legend=dict(
    orientation="h",
    yanchor="bottom",
    y=1.02,
    xanchor="right",
    x=1)
)


fig.write_image("./Graphs/Figure1.png")


"""
## Policy tracker
"""
tracker = pd.read_json(r"./Dataset/iea_policies.json")
tracker = tracker[['policyID', 'year', 'status', 
       'ISO3', 'country', 'type']]
unique_strings = pd.Series([item for sublist in tracker["type"] for item in sublist]).unique()

for element in unique_strings:
    tracker[element] = tracker['type'].apply(lambda x: 1 if element in x else 0)
    tracker[element] = tracker[element].replace(1, element)

tracker_grouped = tracker.groupby(['ISO3', 'Strategic plans', 'year'])['policyID'].count().reset_index()
tracker_grouped = tracker_grouped[tracker_grouped['Strategic plans'] != 0]


# 1. Find the minimum and maximum year for each ISO3
min_year = tracker_grouped['year'].min()
max_year = tracker_grouped['year'].max()

# 2. Generate a complete sequence of years for each ISO3 group
all_years = pd.DataFrame({'year': range(int(min_year), int(max_year) + 1)})

# 3. Merge with the original DataFrame to identify missing years
df_complete = tracker_grouped.set_index('year').groupby('ISO3').apply(
    lambda group: all_years.merge(group, on='year', how='left')
).reset_index(drop=True)
df_complete = df_complete.query('year>2015')
df_complete['ISO3'] = df_complete['ISO3'].ffill()  # or bfill(), depends on your needs
df_complete['ISO3'] = df_complete['ISO3'].bfill()  # or bfill(), depends on your needs

df_complete = df_complete.drop_duplicates(subset=['year', 'ISO3'])

df_complete['policyID'] = df_complete['policyID'].fillna(0)

df_complete.sort_values(by=['year'], inplace=True)

df_complete['cumsum'] = df_complete.groupby('ISO3')['policyID'].transform(pd.Series.cumsum)

tracker = pd.read_json(r"./Dataset/iea_policies.json")
tracker = tracker[['policyID', 'year', 'status', 
       'ISO3', 'country', 'type']]

tracker = tracker.groupby(['ISO3', 'year'])['policyID'].count().reset_index()
tracker = tracker.groupby(['ISO3', 'year'])['policyID'].sum().reset_index()


# 1. Find the minimum and maximum year for each ISO3
min_year = tracker['year'].min()
max_year = tracker['year'].max()

# 2. Generate a complete sequence of years for each ISO3 group
all_years = pd.DataFrame({'year': range(int(min_year), int(max_year) + 1)})

# 3. Merge with the original DataFrame to identify missing years
df_complete = tracker.set_index('year').groupby('ISO3').apply(
    lambda group: all_years.merge(group, on='year', how='left')
).reset_index(drop=True)
df_complete = df_complete.query('year>2015')
df_complete['ISO3'] = df_complete['ISO3'].ffill()  # or bfill(), depends on your needs
df_complete['ISO3'] = df_complete['ISO3'].bfill()  # or bfill(), depends on your needs

df_complete = df_complete.drop_duplicates(subset=['year', 'ISO3'])

df_complete['policyID'] = df_complete['policyID'].fillna(0)

df_complete.sort_values(by=['year'], inplace=True)

df_complete['cumsum'] = df_complete.groupby('ISO3')['policyID'].transform(pd.Series.cumsum)

df_2024 = df_complete[df_complete['year'] == 2024]

# Step 2: Identify countries with cumsum > 10 in 2024
countries_above_10 = df_2024[df_2024['cumsum'] > 10]['ISO3'].unique()

# Step 3: Create the 'highlight' column with 1 for those countries
df_complete['highlight'] = df_complete.apply(lambda row: 1 if row['ISO3'] in countries_above_10 else 0, axis=1)

# Shades of gray
GREY10 = "#1a1a1a"
GREY30 = "#4d4d4d"
GREY40 = "#666666"
GREY50 = "#7f7f7f"
GREY60 = "#999999"
GREY75 = "#bfbfbf"
GREY91 = "#e8e8e8"
GREY98 = "#fafafa"

# Colors used to shade countries
COLOR_SCALE = [
    "#7F3C8D",  # ARG
    "#11A579",  # BRA
    "#3969AC",  # CAN
    "#F2B701",  # CHL
    "#E73F74",  # NAM
    "#80BA5A",  # NGA
    "#E68310",  # CHN
    "#008695",  # SWE
    "#CF1C90",  # MYS
    
 
]
# Vertical lines every 5 years
VLINES = np.arange(2016, 2025, 5)


# Initialize layout ----------------------------------------------
fig, ax = plt.subplots(figsize = (14, 8.5))

# Background color
fig.patch.set_facecolor(GREY98)
ax.set_facecolor(GREY98)

# Vertical lines used as scale reference
for h in VLINES:
    ax.axvline(h, color=GREY91, lw=0.6, zorder=0)

# Horizontal lines
ax.hlines(y=np.arange(0, 15), xmin=2016, xmax=2024, color=GREY91, lw=0.6)

# Darker horizontal line at y=0
ax.hlines(y=0, xmin=2016, xmax=2024, color=GREY60, lw=0.8)

# Add lines ------------------------------------------------------
# Create one data frame for the highlighted countries, and other
# for non-highlighted countries.
df_highlight = df_complete[df_complete["highlight"] == 1]
df_others = df_complete[df_complete["highlight"] == 0]

for group in df_others["ISO3"].unique():
    data = df_others[df_others["ISO3"] == group]
    ax.plot("year", "cumsum", c=GREY75, lw=1.2, alpha=0.5, data=data)

for idx, group in enumerate(df_highlight["ISO3"].unique()):
    data = df_highlight[df_highlight["ISO3"] == group]
    color = COLOR_SCALE[idx]
    ax.plot("year", "cumsum", color=color, lw=1.8, data=data)


#First, adjust axes limits so annotations fit in the plot
ax.set_xlim(2016, 2024.5)
ax.set_ylim(0, 50)

# Positions
LABEL_Y = [
    32,  # CAN
    11,  # CHN
    15,  # GBR
    19,  # FRA
    13,  # JPN
    30,  # EUR
    38,  # AUS
    43,  # USA
    14,  # KOR
]

group = df_highlight["ISO3"].unique()


x_start = 2024
x_end = 2024.5
PAD = 0.1

# Add labels for highlighted countries honly
for idx, group in enumerate(df_highlight["ISO3"].unique()):
    data = df_highlight[(df_highlight["ISO3"] == group) & (df_highlight["year"] == 2024)]
    color = COLOR_SCALE[idx]
    
    # Country name
    text = data["ISO3"].values[0]
    
    # Vertical start of line
    y_start = data["cumsum"].values[0]
    # Vertical end of line
    y_end = LABEL_Y[idx]
    

    
    # Add country text
    ax.text(
        x_end, 
        y_end, 
        text, 
        color=color, 
        fontsize=12, 
        weight="bold", 
        fontfamily="Arial", 
        va="center"
    )

# Customize axes labels and ticks --------------------------------
ax.set_yticks([y for y in np.arange(0, 50, 2)])
ax.set_yticklabels(
    [y for y in np.arange(0, 50, 2)], 
    fontname="Arial", 
    fontsize=11,
    weight=500,
    color=GREY40
)

ax.set_xticks([x for x in np.arange(2016, 2025, 2)])
ax.set_xticklabels(
    [x for x in np.arange(2016, 2025, 2)], 
    fontname= "Arial", 
    fontsize=13,
    weight=500,
    color=GREY40
)



# Increase size and change color of axes ticks
ax.tick_params(axis="x", length=12, color=GREY91)
ax.tick_params(axis="y", length=8, color=GREY91)

# Customize spines
ax.spines["left"].set_color(GREY91)
ax.spines["bottom"].set_color(GREY91)
ax.spines["right"].set_color("none")
ax.spines["top"].set_color("none")

# Add titles, subtitles, and caption -----------------------------
# This uses `fig.text()` instead of regular titles to have full
# control of the text alignment.
subtitle = [
   "The Critical Minerals Policy Tracker developed by the IEA records the key policies implemented in each country related to critical minerals needed for the energy transition. It includes over 35 countries and 450 polices, with the expansion", 
    "focusing on the third policy area on reducing environmental, social and governance impacts. Though not exhaustive, it aims to offer a valuable overview of evolving mineral supply chain governance." 

]


fig.text(
    0.08,
    0.97,
    "Number of policies targeting critical minerals by country",
    color=GREY10,
    fontsize=15,
    fontname="Arial",
    weight="bold"
)

fig.text(
    0.08,
    0.91,
    "\n".join(subtitle),
    ha="left",
    color=GREY30,
    fontname="Arial",
    fontsize=9,
)

fig.text(
    0.08, 
    0.05, 
    " Data from the IEA Critical Minerals Policy Tracker  •  The graph shows the evolution of the number of critical minerals policies implemented by the major economies and mining countries.", 
    fontname="Arial", 
    fontsize=6.5, 
    color=GREY30, 
    ha="left"
)

plt.savefig('./Graphs/Figure2.png')
#fig




"""
## Mineral / critical mining assets
"""

# List of minerals
mineral_list = [
    'Nickel', 'Platinum', 'Copper', 'Palladium', 'Cobalt', 'Chromite',
    'Antimony', 'Zinc', 'Lead', 'Silver', 'Tungsten', 'Iron Ore', 'Tin',
    'Graphite', 'Niobium', 'Lanthanides', 'Lithium', 'Tantalum',
    'Vanadium', 'Bauxite', 'Manganese'
]

# Load and filter the complete dataset
complete = pd.read_csv(r"./Dataset/S&P/Complete.csv")
complete = complete.loc[complete['Year'] == 2022]
complete = complete.loc[complete['ACTUAL_CLOSURE_YR'].isnull()]
complete = complete.drop_duplicates(subset=['PROP_ID'])
complete = complete[complete.PRIMARY_COMMODITY.isin(mineral_list)]

# Group by primary commodity and country
complete_country = complete.groupby(['PRIMARY_COMMODITY', "COUNTRY_NAME"])['PROP_ID'].count()

# Group by primary commodity to get active mines
complete_group = complete.groupby('PRIMARY_COMMODITY')['PROP_ID'].count().reset_index()
complete_group = complete_group.rename(columns={'PRIMARY_COMMODITY': 'Mineral', 'PROP_ID': 'Active mines'})

# Load and filter the projects dataset
proj = pd.read_excel(r"./Dataset/S&P/Projects/prodandreserve_proj-clean.xlsx")
proj = proj.drop_duplicates(subset=['PROP_ID'])
proj = proj[proj.mineral.isin(mineral_list)]

# Group by mineral to get projects
proj_grouped = proj.groupby('mineral')['PROP_ID'].count().reset_index()
proj_grouped = proj_grouped.rename(columns={'mineral': 'Mineral', 'PROP_ID': 'Projects'})

# Merge the active mines and projects data
assets = pd.merge(complete_group, proj_grouped, on='Mineral', how='left')

#Remove antimony
assets = assets.query('Mineral!="Antimony"')

# Save the result to an Excel file
assets.to_excel("./Tables/mining_assets.xlsx")



"""## Where investments go?"""


#### LO investments

# Initialize an empty list to store dataframes
dfs = []

# Process each file in the production_files directory
for filename in os.listdir(production_files):
    try:
        file_path = os.path.join(production_files, filename)
        z = pd.read_excel(file_path)
        z = pd.merge(z, processed, how='left', on='PROP_ID')
        z = z.loc[z['year'] == 2022]
        hhicompany = makeHHI_company_invest(z)
        match = re.search(r'([A-Za-z]+)(?:_LT|\.xlsx)', filename)
        if match:
            x = match.group(1)
            hhicompany['mineral'] = x
            dfs.append(hhicompany)
            print(x)
    except Exception as e:
        print(f"Error processing file {filename}: {e}")

# Load the final companies data
final_companies = pd.read_csv(r"./Dataset/S&P/Company_country/company_master_file.csv")

# Combine all dataframes in dfs
df_combine = pd.concat(dfs, ignore_index=True)
df_combine = pd.merge(df_combine, coord, on='PROP_ID', how='left')
df_combine = pd.merge(df_combine, final_companies, on='OWNER_NAME', how='left')

# Filter and group data for specific queries
test_lo = df_combine.query('OWNER_COUNTRY=="USA" | OWNER_COUNTRY=="China" ')
test_lo = test_lo.query('mineral=="Copper"')
test_lo = test_lo.groupby(['OWNER_COUNTRY', 'COUNTRY_NAME'])['value'].sum().reset_index()
test_lo = df_combine.query('mineral=="Copper"')

# Create a dictionary for the price data
data = {
    'mineral': [
        'Bauxite', 'Chromite', 'Cobalt', 'Copper', 'Iron', 'Lead', 'Lithium', 'Manganese',
        'Molybdenum', 'Nickel', 'Palladium', 'Platinum', 'Lanthanides', 'Silver', 'Tin',
        'Uranium', 'Vanadium', 'Zinc'
    ],
    'Price': [
        '2706.988189', '10826.34276', '63764.7193', '8828.916814', '120.7', '2152.724984',
        '416957.7632', '4501.48339', '39173.00767', '25867.26271', '2107.402924',
        '960.877748', '10021.89129', '21.77085785', '31383.28668', '40.77293139',
        '23894.90852', '3484.86529'
    ]
}

# Convert the price data to a dataframe
price = pd.DataFrame(data)
price['Price'] = price['Price'].astype(float)

# Initialize an empty list to store foreign influence dataframes
dfs = []

# Process each mineral in the mineral_list
for mineral in mineral_list:
    test = make_foreign_influence(mineral, df_combine)
    dfs.append(test)
    print(mineral)

# Combine all foreign influence dataframes
df_final = pd.concat(dfs, ignore_index=True)

# Calculate the mean share of foreign influence
df_final_mean = df_final.groupby(['COUNTRY_NAME', 'National'])['Share_foreign'].mean().reset_index()
df_final_mean.sort_values(by=['National', 'Share_foreign'], ascending=False, inplace=True)
df_final_mean['National'] = df_final_mean['National'].map({
    "Foreign control": "Share of the mining sector owned by a foreign mining company",
    "National control": "Share of the mining sector owned by the state or a national mining company"
})


fig = px.bar(df_final_mean, x="COUNTRY_NAME", y="Share_foreign", color="National", #title="National vs foreign control (lo) ",
             labels={
                     "Share_foreign": "",
                     "COUNTRY_NAME": ""})

fig.update_layout(
    legend_title='',
    template='plotly_white',  # Use a cleaner theme
    width=1800,  # Adjust the width
    height=800,  # Adjust the height
    xaxis = dict(
      tickfont = dict(size=14)),
    yaxis = dict(
      tickfont = dict(size=14)),
    legend=dict(
    #orientation="h",
    yanchor="bottom",
    y=1.00,
    xanchor="right",
    x=1,
    title_font_family="Times New Roman",
                              font=dict(size= 16))
)

fig.write_image("./Graphs/Figure5.png")




"""#### SOC investments"""

# Load the final companies data
final_companies = pd.read_csv(r"./Dataset/S&P/Company_country/company_master_file.csv")

# Initialize an empty list to store dataframes
dfs = []

# Load and rename columns in the ownership data
owner_ten = pd.read_excel(r"./Dataset/S&P/Ownership_ten.xls", header=3)
owner_ten = owner_ten.rename(columns={'COMPANY_NAME': 'OWNER_NAME'})

# Load and rename columns in the master company data
master_company = pd.read_csv(r"./Dataset/S&P/Company_country/company_master_file.csv")
master_company = master_company.rename(columns={'OWNER_NAME': 'Financial_name', 'OWNER_COUNTRY': 'Country'})

# Process each file in the production_files directory

for element in os.listdir(production_files):
  try:
    file_path = os.path.join(production_files, element)
    z = pd.read_excel(file_path)
    z = pd.merge(z, processed, how='left', on='PROP_ID')
    z = pd.merge(z, coord, on='PROP_ID', how='left')
    z = z.loc[z['year'] == 2022]
    z['Share_1'] = z['value']*(z["OWNER_PCT"]/100)
    z['Share_2'] = z['value']*(z["OWNER_PCT.1"]/100)
    z['Share_3'] = z['value']*(z["OWNER_PCT.2"]/100)
    z['Share_4'] = z['value']*(z["OWNER_PCT.3"]/100)
    z['Share_5'] = z['value']*(z["OWNER_PCT.4"]/100)
    z['Share_6'] = z['value']*(z["OWNER_PCT.5"]/100)
    z['Share_7'] = z['value']*(z["OWNER_PCT.6"]/100)
    z['Share_8'] = z['value']*(z["OWNER_PCT.7"]/100)
    z['Share_9'] = z['value']*(z["OWNER_PCT.8"]/100)
    z['Share_10'] = z['value']*(z["OWNER_PCT.9"]/100)

    df_share_melted = z[['PROP_ID', 'Share_1', 'Share_2',
                'Share_3', 'Share_4', 'Share_5','Share_6', 'Share_7', 'Share_8', 'Share_9', 'Share_10']]
    df_share_melted = pd.melt(df_share_melted, id_vars =['PROP_ID'], value_vars =['Share_1', 'Share_2',
            'Share_3', 'Share_4', 'Share_5','Share_6', 'Share_7', 'Share_8', 'Share_9', 'Share_10'])

    df_share_melted = df_share_melted.dropna(subset=['value'])
    df_share_oc = z[['PROP_ID', 'OWNER_NAME', 'OWNER_NAME.1', 'OWNER_NAME.2', 'OWNER_NAME.3',
            'OWNER_NAME.4',  'OWNER_NAME.5', 'OWNER_NAME.6', 'OWNER_NAME.7',
            'OWNER_NAME.8', 'OWNER_NAME.9']]
    df_share_oc = pd.melt(df_share_oc, id_vars =['PROP_ID'], value_vars =['OWNER_NAME', 'OWNER_NAME.1', 'OWNER_NAME.2', 'OWNER_NAME.3',
            'OWNER_NAME.4',  'OWNER_NAME.5', 'OWNER_NAME.6', 'OWNER_NAME.7',
            'OWNER_NAME.8', 'OWNER_NAME.9'])
    df_share_oc['variable'] = df_share_oc['variable'].replace(['OWNER_NAME', 'OWNER_NAME.1', 'OWNER_NAME.2', 'OWNER_NAME.3',
            'OWNER_NAME.4',  'OWNER_NAME.5', 'OWNER_NAME.6', 'OWNER_NAME.7',
            'OWNER_NAME.8', 'OWNER_NAME.9'], ['Share_1', 'Share_2',
                    'Share_3', 'Share_4', 'Share_5','Share_6', 'Share_7', 'Share_8', 'Share_9', 'Share_10'])
    df_share_combine = pd.merge(df_share_melted, df_share_oc, how='left', on=['PROP_ID', 'variable'])
    df_share_combine = pd.merge(df_share_combine, coord, how='left', on=['PROP_ID'])
    df_share_combine = df_share_combine.rename(columns={'value_x': 'value', 'value_y': 'OWNER_NAME'})



    # Introduce the inst owner dataset

    df_share_combine = pd.merge(df_share_combine, owner_ten, on='OWNER_NAME', how='left')

    # Correct for mining investor that are already the Ultimate Owner
    df_share_combine['1ST_INST_OWNER_NAME'] = df_share_combine['1ST_INST_OWNER_NAME'].fillna(df_share_combine['OWNER_NAME'])
    df_share_combine['1ST_INST_OWNER_PCT'] = df_share_combine['1ST_INST_OWNER_PCT'].fillna(100)

    z = df_share_combine

    #Rearrange the shares of institutional owners to 100% scale
    z = rearrange_shares(z)

    z['Share_1'] = z['value']*(z["1ST_INST_OWNER_PCT"]/100)
    z['Share_2'] = z['value']*(z["2ND_INST_OWNER_PCT"]/100)
    z['Share_3'] = z['value']*(z["3RD_INST_OWNER_PCT"]/100)
    z['Share_4'] = z['value']*(z["4TH_INST_OWNER_PCT"]/100)
    z['Share_5'] = z['value']*(z["5TH_INST_OWNER_PCT"]/100)
    z['Share_6'] = z['value']*(z["6TH_INST_OWNER_PCT"]/100)
    z['Share_7'] = z['value']*(z["7TH_INST_OWNER_PCT"]/100)
    z['Share_8'] = z['value']*(z["8TH_INST_OWNER_PCT"]/100)
    z['Share_9'] = z['value']*(z["9TH_INST_OWNER_PCT"]/100)
    z['Share_10'] = z['value']*(z["10TH_INST_OWNER_PCT"]/100)

    z_ownername = z[['PROP_ID', 'OWN_STRUCTURE']]
    z_ownername = z_ownername.drop_duplicates()

    z_inst_share = z[['PROP_ID', 'Share_1', 'Share_2',
          'Share_3', 'Share_4', 'Share_5', 'Share_6', 'Share_7', 'Share_8',
          'Share_9', 'Share_10']]

    z_inst_share = pd.melt(z_inst_share, id_vars =['PROP_ID'], value_vars =['Share_1', 'Share_2',
          'Share_3', 'Share_4', 'Share_5', 'Share_6', 'Share_7', 'Share_8',
          'Share_9', 'Share_10'])
    z_inst_share = z_inst_share.rename(columns={'variable': 'share_rank'})
    z_inst_share = z_inst_share[['PROP_ID', 'value']]

    z_inst_name = z[['PROP_ID', '1ST_INST_OWNER_NAME', '2ND_INST_OWNER_NAME',
            '3RD_INST_OWNER_NAME','4TH_INST_OWNER_NAME', '5TH_INST_OWNER_NAME',
            '6TH_INST_OWNER_NAME', '7TH_INST_OWNER_NAME', '8TH_INST_OWNER_NAME',
            '9TH_INST_OWNER_NAME', '10TH_INST_OWNER_NAME']]

    z_inst_name = pd.melt(z_inst_name, id_vars =[('PROP_ID')], value_vars =['1ST_INST_OWNER_NAME', '2ND_INST_OWNER_NAME',
            '3RD_INST_OWNER_NAME','4TH_INST_OWNER_NAME', '5TH_INST_OWNER_NAME',
            '6TH_INST_OWNER_NAME', '7TH_INST_OWNER_NAME', '8TH_INST_OWNER_NAME',
            '9TH_INST_OWNER_NAME', '10TH_INST_OWNER_NAME'])
    z_inst_name = z_inst_name.rename(columns={'variable': 'name_rank', 'value':'Financial_name'})

    z_inst_name = pd.merge(z_inst_name, country_name, on='Financial_name', how='left')
    z_inst_name["Country"] = z_inst_name["Country"].replace(["United States"], "USA")
    z_inst_name["Country"] = z_inst_name["Country"].replace(["Hong Kong"], "China")
    z_inst_name["Country"] = z_inst_name["Country"].replace(["Hong Kong"], "China")



    df_combine = pd.concat([z_inst_share, z_inst_name],  axis="columns")

    df_combine = pd.merge(df_combine, master_company, on='Financial_name', how='left', suffixes=('', '_new'))

    df_combine['Country'] = df_combine['Country'].combine_first(df_combine['Country_new'])
    df_combine.drop(columns=['Country_new'], inplace=True)
    df_combine['Country'] = df_combine.apply(lambda row: fill_missing_countries(row, missing_countries), axis=1)

    mineral_name = re.sub(r'\.xlsx$', '', element)
    df_combine['mineral'] = mineral_name
    duplicated_columns = df_combine.columns[df_combine.columns.duplicated()]

    # Dropping the duplicated columns except the first occurrence
    df_combine = df_combine.loc[:, ~df_combine.columns.duplicated()]


    dfs.append(df_combine)
  except:
    pass

df_combine = pd.concat(dfs, ignore_index=True)
combine_coord =  pd.merge(df_combine, coord, on='PROP_ID', how='inner')

dfs=[]
for mineral in mineral_list:
  test = make_foreign_influence_soc(mineral, combine_coord, price)
  dfs.append(test)
  print(mineral)
df_final = pd.concat(dfs, ignore_index=True)

df_final_mean = df_final.groupby(['COUNTRY_NAME', 'National'])['Share_foreign'].mean().reset_index()
df_final_mean.sort_values(by=['National','Share_foreign'],  ascending=False, inplace=True)
df_final_mean['National'] =df_final_mean['National'].map({"Foreign control":"Share of the mining sector owned by foreign equity ownership","National control":"Share of the mining sector owned by domestic equity ownership"})


fig = px.bar(df_final_mean, x="COUNTRY_NAME", y="Share_foreign", color="National", #title="National vs foreign control (soc) ",
             labels={
                     "Share_foreign": "",
                     "COUNTRY_NAME": ""})

fig.update_layout(
    legend_title='',
    template='plotly_white',  # Use a cleaner theme
    width=1800,  # Adjust the width
    height=800,  # Adjust the height
    xaxis = dict(
      tickfont = dict(size=14)),
    yaxis = dict(
      tickfont = dict(size=14)),
    legend=dict(
    #orientation="h",
    yanchor="bottom",
    y=1.00,
    xanchor="right",
    x=1,
    title_font_family="Times New Roman",
                              font=dict(size= 16))
)
fig.write_image("./Graphs/Figure6.png")


"""#### Figure 10 """


# Filter and group data for specific queries
test_soc = combine_coord.query('Country=="USA" | Country=="China" ')
test_soc = test_soc.query('mineral=="Copper"')
test_soc = test_soc.groupby(['Country', 'COUNTRY_NAME'])['value'].sum().reset_index()
test_soc = test_soc.rename(columns={'Country': 'OWNER_COUNTRY'})
test_soc['Method'] = "soc"

# Combine the data with test_lo
test_lo['Method'] = "lo"
test_combine = pd.concat([test_lo, test_soc], ignore_index=True)
test_combine['ID'] = test_combine['OWNER_COUNTRY'] + "_" + test_combine['Method']

# Group and pivot the combined data
test_combine = test_combine.groupby(['COUNTRY_NAME', 'ID'])['value'].sum().reset_index()
finalform = test_combine.pivot(index='COUNTRY_NAME', columns='ID', values='value')
finalform = finalform.fillna(0)

# Calculate the total and adjust values
finalform['TOTAL'] = finalform['China_lo'] + finalform['USA_lo'] + finalform['China_soc'] + finalform['USA_soc']
finalform['China_lo'] = finalform['China_lo'] * (-1)
finalform['USA_lo'] = finalform['USA_lo'] * (-1)
finalform = finalform.sort_values(by=['TOTAL'], ascending=True)

# Keep only subsample, adapt if needed
finalform = finalform.loc[finalform.index.isin([
    "Peru", "Chile", "China", "USA", "Dem. Rep. Congo", "Mexico", "Indonesia",
    "Australia", "Zambia", "Serbia", "Canada", "Panama", "Brazil", "Poland"
])]

import matplotlib.pyplot as plt

# Your existing code but modified to use horizontal bar plots
fig, ax = plt.subplots(1, figsize=(16, 8))

# Plotting the horizontal bar plots
ax.barh(finalform.index, finalform['USA_soc'], color = '#5E96E9', height = 0.5)
ax.barh(finalform.index, finalform['China_soc'], left = finalform['USA_soc'], color = '#ff80be', height = 0.5)
ax.barh(finalform.index, finalform['China_lo'], color = '#ff80be', height = 0.5)
ax.barh(finalform.index, finalform['USA_lo'], left = finalform['China_lo'], color = '#5E96E9', height = 0.5)

# Remove spines
ax.spines['right'].set_visible(False)
ax.spines['left'].set_visible(False)
ax.spines['top'].set_visible(False)
ax.spines['bottom'].set_visible(False)

# Grid
ax.set_axisbelow(True)
ax.xaxis.grid(color='gray', linestyle='dashed', alpha=0.7)

# X ticks (commented out as in your original code)
# xticks_labels = ['Jan', 'Fev', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov']
# plt.xticks(finalform.index , labels = xticks_labels)

# Title and legend
legend_label = ['USA', 'China']
plt.legend(legend_label, ncol = 4, bbox_to_anchor=([1, 1.05, 0, 0]), frameon = False)
plt.title('Copper - China vs USA\n', loc='right')

xlim = ax.get_xlim()
positive_part_x = (xlim[0] + xlim[1]) / 2
negative_part_x = xlim[0] / 2
ax.text(positive_part_x, len(finalform) + 0.5, ' Equity control', ha='center', va='bottom', fontsize=12, fontweight='bold')
ax.text(negative_part_x, len(finalform) + 0.5, 'Direct ownership control', ha='center', va='bottom', fontsize=12, fontweight='bold')

ax.set_xticklabels([str(abs(x)) for x in ax.get_xticks()])

# Show the plot
plt.savefig('./Graphs/Figure10.png')
