import pandas as pd
import pycountry
import os
import numpy as np



def processing_assets (folder_path):
# Filter for Excel files 
    # List all files in the folder
    files = os.listdir(folder_path)
    excel_files = [file for file in files if file.endswith('.xlsx')]

    dfs = []


    # Loop over each Excel file and process them
    for file in excel_files:
        file_path = os.path.join(folder_path, file)
        df = pd.read_excel(file_path)
        df = df[['PROP_ID',
        'OWNER_NAME', 'OWNER_NAME.1', 'OWNER_NAME.2', 'OWNER_NAME.3',
        'OWNER_NAME.4',  'OWNER_NAME.5', 'OWNER_NAME.6', 'OWNER_NAME.7',
        'OWNER_NAME.8', 'OWNER_NAME.9', 'OWNER_PCT',
        'OWNER_PCT.1', 'OWNER_PCT.2', 'OWNER_PCT.3', 'OWNER_PCT.4',
        'OWNER_PCT.5', 'OWNER_PCT.6', 'OWNER_PCT.7', 'OWNER_PCT.8',
        'OWNER_PCT.9', 'OWNER_COUNTRY', 'OWNER_COUNTRY.1', 'OWNER_COUNTRY.2',
        'OWNER_COUNTRY.3', 'OWNER_COUNTRY.4', 'OWNER_COUNTRY.5', 'OWNER_COUNTRY.6',
        'OWNER_COUNTRY.7', 'OWNER_COUNTRY.8', 'OWNER_COUNTRY.9'
        ]]
        dfs.append(df)
    combined_df = pd.concat(dfs)
    # Remove duplicate rows
    unique_df = combined_df.drop_duplicates()

    # Reset the index of the unique DataFrame
    unique_df = unique_df.reset_index(drop=True)
    return unique_df


def loopoverreserves(unique_df, folder_path):
    for filename in os.listdir(folder_path):
        try:
            file_path = os.path.join(folder_path, filename)
            z = pd.read_excel(file_path)
            z = pd.merge(z, unique_df, how='left', on='PROP_ID')
            print(filename)
            return z
        except:
            pass





def makeHHI_country(z):
    z['Share_1'] = z['value']*(z["OWNER_PCT"]/100)
    z['Share_2'] = z['value']*(z["OWNER_PCT.1"]/100)
    z['Share_3'] = z['value']*(z["OWNER_PCT.2"]/100)
    z['Share_4'] = z['value']*(z["OWNER_PCT.3"]/100)
    z['Share_5'] = z['value']*(z["OWNER_PCT.4"]/100)
    z['Share_6'] = z['value']*(z["OWNER_PCT.5"]/100)
    z['Share_7'] = z['value']*(z["OWNER_PCT.6"]/100)
    z['Share_8'] = z['value']*(z["OWNER_PCT.7"]/100)
    z['Share_9'] = z['value']*(z["OWNER_PCT.8"]/100)
    z['Share_10'] = z['value']*(z["OWNER_PCT.9"]/100)


    df_share_melted = z[['PROP_ID', 'Share_1', 'Share_2',
            'Share_3', 'Share_4', 'Share_5','Share_6', 'Share_7', 'Share_8', 'Share_9', 'Share_10']]
    df_share_melted = pd.melt(df_share_melted, id_vars =['PROP_ID'], value_vars =['Share_1', 'Share_2',
        'Share_3', 'Share_4', 'Share_5','Share_6', 'Share_7', 'Share_8', 'Share_9', 'Share_10'])

    df_share_melted = df_share_melted.dropna(subset=['value'])

    df_share_oc = z[['PROP_ID', 'OWNER_COUNTRY',
                'OWNER_COUNTRY.1', 'OWNER_COUNTRY.2', 'OWNER_COUNTRY.3',
                'OWNER_COUNTRY.4', 'OWNER_COUNTRY.5',
                'OWNER_COUNTRY.6', 'OWNER_COUNTRY.7', 'OWNER_COUNTRY.8',
                'OWNER_COUNTRY.9']]

    df_share_oc = pd.melt(df_share_oc, id_vars =['PROP_ID'], value_vars =['OWNER_COUNTRY',
                'OWNER_COUNTRY.1', 'OWNER_COUNTRY.2', 'OWNER_COUNTRY.3',
                'OWNER_COUNTRY.4', 'OWNER_COUNTRY.5',
                'OWNER_COUNTRY.6', 'OWNER_COUNTRY.7', 'OWNER_COUNTRY.8',
                'OWNER_COUNTRY.9'])
    df_share_oc['variable'] = df_share_oc['variable'].replace(['OWNER_COUNTRY',
                'OWNER_COUNTRY.1', 'OWNER_COUNTRY.2', 'OWNER_COUNTRY.3',
                'OWNER_COUNTRY.4', 'OWNER_COUNTRY.5',
                'OWNER_COUNTRY.6', 'OWNER_COUNTRY.7', 'OWNER_COUNTRY.8',
                'OWNER_COUNTRY.9'], ['Share_1', 'Share_2',
                'Share_3', 'Share_4', 'Share_5','Share_6', 'Share_7', 'Share_8', 'Share_9', 'Share_10'])
    df_share_combine = pd.merge(df_share_melted, df_share_oc, how='left', on=['PROP_ID', 'variable'])
    df_share_combine = df_share_combine.rename(columns={'value_x': 'value', 'value_y': 'OWNER_COUNTRY'})

    df_share_combine['OWNED_MINES'] = df_share_combine.groupby('OWNER_COUNTRY')['OWNER_COUNTRY'].transform('count')
    test = df_share_combine.groupby("OWNER_COUNTRY").agg({'value': 'sum', 'OWNED_MINES': 'first'})
    df = test
        #df = test.to_frame().reset_index()
    df['total'] = df['value']/df['value'].sum() * 100

    df = df.sort_values(by=['total'], ascending=False)

    df['Perc_mines'] = (df['OWNED_MINES']/df['OWNED_MINES'].sum())*100
    df['HHI_mines'] = df['Perc_mines']*df['Perc_mines']
    df['HHI_reserves'] = df['total'] * df['total']
    new_row = {'OWNER_COUNTRY':'All',
            'value':df['value'].sum(),
            'OWNED_MINES':df['OWNED_MINES'].sum(),
            'total': df['total'].sum()   ,
            'Perc_mines':df['Perc_mines'].sum(),
            'HHI_mines': df['HHI_mines'].sum(),
            'HHI_reserves': df['HHI_reserves'].sum() }
    new_row = pd.DataFrame([new_row])
    df = df.reset_index()
    df_country = pd.concat([df, new_row], ignore_index=True)
    df_country.sort_values(by='HHI_reserves', ascending=False, inplace=True)
    return df_country



def makeHHI_company_invest(z):
    z['Share_1'] = z['value']*(z["OWNER_PCT"]/100)
    z['Share_2'] = z['value']*(z["OWNER_PCT.1"]/100)
    z['Share_3'] = z['value']*(z["OWNER_PCT.2"]/100)
    z['Share_4'] = z['value']*(z["OWNER_PCT.3"]/100)
    z['Share_5'] = z['value']*(z["OWNER_PCT.4"]/100)
    z['Share_6'] = z['value']*(z["OWNER_PCT.5"]/100)
    z['Share_7'] = z['value']*(z["OWNER_PCT.6"]/100)
    z['Share_8'] = z['value']*(z["OWNER_PCT.7"]/100)
    z['Share_9'] = z['value']*(z["OWNER_PCT.8"]/100)
    z['Share_10'] = z['value']*(z["OWNER_PCT.9"]/100)


    df_share_melted = z[['PROP_ID', 'Share_1', 'Share_2',
            'Share_3', 'Share_4', 'Share_5','Share_6', 'Share_7', 'Share_8', 'Share_9', 'Share_10']]
    df_share_melted = pd.melt(df_share_melted, id_vars =['PROP_ID'], value_vars =['Share_1', 'Share_2',
        'Share_3', 'Share_4', 'Share_5','Share_6', 'Share_7', 'Share_8', 'Share_9', 'Share_10'])

    df_share_melted = df_share_melted.dropna(subset=['value'])

    df_share_oc = z[['PROP_ID', 'OWNER_NAME', 'OWNER_NAME.1', 'OWNER_NAME.2', 'OWNER_NAME.3',
        'OWNER_NAME.4',  'OWNER_NAME.5', 'OWNER_NAME.6', 'OWNER_NAME.7',
        'OWNER_NAME.8', 'OWNER_NAME.9']]
    df_share_oc = pd.melt(df_share_oc, id_vars =['PROP_ID'], value_vars =['OWNER_NAME', 'OWNER_NAME.1', 'OWNER_NAME.2', 'OWNER_NAME.3',
        'OWNER_NAME.4',  'OWNER_NAME.5', 'OWNER_NAME.6', 'OWNER_NAME.7',
        'OWNER_NAME.8', 'OWNER_NAME.9'])
    df_share_oc['variable'] = df_share_oc['variable'].replace(['OWNER_NAME', 'OWNER_NAME.1', 'OWNER_NAME.2', 'OWNER_NAME.3',
        'OWNER_NAME.4',  'OWNER_NAME.5', 'OWNER_NAME.6', 'OWNER_NAME.7',
        'OWNER_NAME.8', 'OWNER_NAME.9'], ['Share_1', 'Share_2',
                'Share_3', 'Share_4', 'Share_5','Share_6', 'Share_7', 'Share_8', 'Share_9', 'Share_10'])
    df_share_combine = pd.merge(df_share_melted, df_share_oc, how='left', on=['PROP_ID', 'variable'])
    df_share_combine = df_share_combine.rename(columns={'value_x': 'value', 'value_y': 'OWNER_NAME'})
    return df_share_combine



def makeHHI_company(z):
    z['Share_1'] = z['value']*(z["OWNER_PCT"]/100)
    z['Share_2'] = z['value']*(z["OWNER_PCT.1"]/100)
    z['Share_3'] = z['value']*(z["OWNER_PCT.2"]/100)
    z['Share_4'] = z['value']*(z["OWNER_PCT.3"]/100)
    z['Share_5'] = z['value']*(z["OWNER_PCT.4"]/100)
    z['Share_6'] = z['value']*(z["OWNER_PCT.5"]/100)
    z['Share_7'] = z['value']*(z["OWNER_PCT.6"]/100)
    z['Share_8'] = z['value']*(z["OWNER_PCT.7"]/100)
    z['Share_9'] = z['value']*(z["OWNER_PCT.8"]/100)
    z['Share_10'] = z['value']*(z["OWNER_PCT.9"]/100)


    df_share_melted = z[['PROP_ID', 'Share_1', 'Share_2',
            'Share_3', 'Share_4', 'Share_5','Share_6', 'Share_7', 'Share_8', 'Share_9', 'Share_10']]
    df_share_melted = pd.melt(df_share_melted, id_vars =['PROP_ID'], value_vars =['Share_1', 'Share_2',
        'Share_3', 'Share_4', 'Share_5','Share_6', 'Share_7', 'Share_8', 'Share_9', 'Share_10'])

    df_share_melted = df_share_melted.dropna(subset=['value'])

    df_share_oc = z[['PROP_ID', 'OWNER_NAME', 'OWNER_NAME.1', 'OWNER_NAME.2', 'OWNER_NAME.3',
        'OWNER_NAME.4',  'OWNER_NAME.5', 'OWNER_NAME.6', 'OWNER_NAME.7',
        'OWNER_NAME.8', 'OWNER_NAME.9']]
    df_share_oc = pd.melt(df_share_oc, id_vars =['PROP_ID'], value_vars =['OWNER_NAME', 'OWNER_NAME.1', 'OWNER_NAME.2', 'OWNER_NAME.3',
        'OWNER_NAME.4',  'OWNER_NAME.5', 'OWNER_NAME.6', 'OWNER_NAME.7',
        'OWNER_NAME.8', 'OWNER_NAME.9'])
    df_share_oc['variable'] = df_share_oc['variable'].replace(['OWNER_NAME', 'OWNER_NAME.1', 'OWNER_NAME.2', 'OWNER_NAME.3',
        'OWNER_NAME.4',  'OWNER_NAME.5', 'OWNER_NAME.6', 'OWNER_NAME.7',
        'OWNER_NAME.8', 'OWNER_NAME.9'], ['Share_1', 'Share_2',
                'Share_3', 'Share_4', 'Share_5','Share_6', 'Share_7', 'Share_8', 'Share_9', 'Share_10'])
    df_share_combine = pd.merge(df_share_melted, df_share_oc, how='left', on=['PROP_ID', 'variable'])
    df_share_combine = df_share_combine.rename(columns={'value_x': 'value', 'value_y': 'OWNER_NAME'})

    df_share_combine['OWNED_MINES'] = df_share_combine.groupby('OWNER_NAME')['OWNER_NAME'].transform('count')
    test = df_share_combine.groupby("OWNER_NAME").agg({'value': 'sum', 'OWNED_MINES': 'first'})
    df = test
        #df = test.to_frame().reset_index()
    df['total'] = df['value']/df['value'].sum() * 100

    df = df.sort_values(by=['total'], ascending=False)
    df['Perc_mines'] = (df['OWNED_MINES']/df['OWNED_MINES'].sum())*100
    df['HHI_mines'] = df['Perc_mines']*df['Perc_mines']
    df['HHI_reserves'] = df['total'] * df['total']
    new_row = {'OWNER_NAME':'All',
            'value':df['value'].sum(),
            'OWNED_MINES':df['OWNED_MINES'].sum(),
            'total': df['total'].sum()   ,
            'Perc_mines':df['Perc_mines'].sum(),
            'HHI_mines': df['HHI_mines'].sum(),
            'HHI_reserves': df['HHI_reserves'].sum() }
    new_row = pd.DataFrame([new_row])
    df = df.reset_index()
    df_company = pd.concat([df, new_row], ignore_index=True)
    df_company.sort_values(by='HHI_reserves', ascending=False, inplace=True)
    return df_company



def makeHHI_country_geo(z):
  z = z.loc[z['year'] == 2022]
  z = z.groupby("COUNTRY_NAME").sum().reset_index()
  z = z[['COUNTRY_NAME', "value"]]
  z['total'] = z['value']/z['value'].sum() * 100
  z = z.sort_values(by=['total'], ascending=False)
  z= z.sort_values(by=['total'], ascending=False)
  z['HHI_reserves'] = z['total'] * z['total']
  new_row = {'COUNTRY_NAME':'All',
              'value':z['value'].sum(),
              'total': z['total'].sum()   ,
              'HHI_reserves': z['HHI_reserves'].sum() }
  new_row = pd.DataFrame([new_row])
  z = z.reset_index()
  df_country = pd.concat([z, new_row], ignore_index=True)
  df_country.sort_values(by='HHI_reserves', ascending=False, inplace=True)
  return df_country



def prep_country(z):
    z['Share_1'] = z['value']*(z["OWNER_PCT"]/100)
    z['Share_2'] = z['value']*(z["OWNER_PCT.1"]/100)
    z['Share_3'] = z['value']*(z["OWNER_PCT.2"]/100)
    z['Share_4'] = z['value']*(z["OWNER_PCT.3"]/100)
    z['Share_5'] = z['value']*(z["OWNER_PCT.4"]/100)
    z['Share_6'] = z['value']*(z["OWNER_PCT.5"]/100)
    z['Share_7'] = z['value']*(z["OWNER_PCT.6"]/100)
    z['Share_8'] = z['value']*(z["OWNER_PCT.7"]/100)
    z['Share_9'] = z['value']*(z["OWNER_PCT.8"]/100)
    z['Share_10'] = z['value']*(z["OWNER_PCT.9"]/100)


    df_share_melted = z[['PROP_ID', 'Share_1', 'Share_2',
            'Share_3', 'Share_4', 'Share_5','Share_6', 'Share_7', 'Share_8', 'Share_9', 'Share_10']]
    df_share_melted = pd.melt(df_share_melted, id_vars =['PROP_ID'], value_vars =['Share_1', 'Share_2',
        'Share_3', 'Share_4', 'Share_5','Share_6', 'Share_7', 'Share_8', 'Share_9', 'Share_10'])

    df_share_melted = df_share_melted.dropna(subset=['value'])

    df_share_oc = z[['PROP_ID', 'OWNER_COUNTRY',
                'OWNER_COUNTRY.1', 'OWNER_COUNTRY.2', 'OWNER_COUNTRY.3',
                'OWNER_COUNTRY.4', 'OWNER_COUNTRY.5',
                'OWNER_COUNTRY.6', 'OWNER_COUNTRY.7', 'OWNER_COUNTRY.8',
                'OWNER_COUNTRY.9']]

    df_share_oc = pd.melt(df_share_oc, id_vars =['PROP_ID'], value_vars =['OWNER_COUNTRY',
                'OWNER_COUNTRY.1', 'OWNER_COUNTRY.2', 'OWNER_COUNTRY.3',
                'OWNER_COUNTRY.4', 'OWNER_COUNTRY.5',
                'OWNER_COUNTRY.6', 'OWNER_COUNTRY.7', 'OWNER_COUNTRY.8',
                'OWNER_COUNTRY.9'])
    df_share_oc['variable'] = df_share_oc['variable'].replace(['OWNER_COUNTRY',
                'OWNER_COUNTRY.1', 'OWNER_COUNTRY.2', 'OWNER_COUNTRY.3',
                'OWNER_COUNTRY.4', 'OWNER_COUNTRY.5',
                'OWNER_COUNTRY.6', 'OWNER_COUNTRY.7', 'OWNER_COUNTRY.8',
                'OWNER_COUNTRY.9'], ['Share_1', 'Share_2',
                'Share_3', 'Share_4', 'Share_5','Share_6', 'Share_7', 'Share_8', 'Share_9', 'Share_10'])
    df_share_combine = pd.merge(df_share_melted, df_share_oc, how='left', on=['PROP_ID', 'variable'])
    df_share_combine = df_share_combine.rename(columns={'value_x': 'value', 'value_y': 'OWNER_COUNTRY'})

    df_share_combine['OWNED_MINES'] = df_share_combine.groupby('OWNER_COUNTRY')['OWNER_COUNTRY'].transform('count')
    test = df_share_combine.groupby("OWNER_COUNTRY").agg({'value': 'sum', 'OWNED_MINES': 'first'})
    return test



# Function to get ISO3 code from country name
def get_iso3(country_name):
    try:
        country = pycountry.countries.search_fuzzy(country_name)
        return country[0].alpha_3
    except LookupError:
        return 'Unknown'



def rearrange_shares(data):
  data.fillna(0, inplace=True)
  sum_perc = data['1ST_INST_OWNER_PCT'] + data['2ND_INST_OWNER_PCT'] + data['3RD_INST_OWNER_PCT'] + data['4TH_INST_OWNER_PCT'] + data['5TH_INST_OWNER_PCT'] + data['6TH_INST_OWNER_PCT'] +  data['7TH_INST_OWNER_PCT'] + data['8TH_INST_OWNER_PCT'] + data['9TH_INST_OWNER_PCT'] + data['10TH_INST_OWNER_PCT']
  scaling_factor = 100/sum_perc
  data['1ST_INST_OWNER_PCT'] = data['1ST_INST_OWNER_PCT'] * scaling_factor
  data['2ND_INST_OWNER_PCT'] = data['2ND_INST_OWNER_PCT'] * scaling_factor
  data['3RD_INST_OWNER_PCT'] = data['3RD_INST_OWNER_PCT'] * scaling_factor
  data['4TH_INST_OWNER_PCT'] = data['4TH_INST_OWNER_PCT'] * scaling_factor
  data['5TH_INST_OWNER_PCT'] = data['5TH_INST_OWNER_PCT'] * scaling_factor
  data['6TH_INST_OWNER_PCT'] = data['6TH_INST_OWNER_PCT'] * scaling_factor
  data['7TH_INST_OWNER_PCT'] = data['7TH_INST_OWNER_PCT'] * scaling_factor
  data['8TH_INST_OWNER_PCT'] = data['8TH_INST_OWNER_PCT'] * scaling_factor
  data['9TH_INST_OWNER_PCT'] = data['9TH_INST_OWNER_PCT'] * scaling_factor
  data['10TH_INST_OWNER_PCT'] = data['10TH_INST_OWNER_PCT'] * scaling_factor
  return data

missing_countries = {
       'Public Investment Fund of Saudi Arabia': 'Saudi Arabia',
       'Cia Minera Caudalosa SA': 'Peru',
       'Sail Natural Resources LP': 'USA',
       'Elite Fund Management Ltd': 'South Africa',
       'Chengde Zhongfu Mining Group Co., Ltd': 'China',
       'Zhangjiakou Caijiaying Lead Zinc Mining Company Limited': 'China',
       'Reynas Minas S.A. de C.V.': 'Mexico',
       'Xinjiang Anshun Dakuangshan Technology Engineering Co., Ltd.': 'China',
       'Shandong Longxing Group Laizhou Chemical Building Materials Co.,Ltd.': 'China',
       'Gubbins Group': 'Australia',
       'Burkina Faso': 'Burkina Faso',
       'Akmetal Madencilik Sanayi Ve Ticaret A.S.': 'Turkey',
       'Minera Tayahua, S.A. de C.V.': 'Mexico',
       'Henan Non-Ferrous Geology Bureau': 'China',
       'Kohlhase & Stower Asset Management GmbH': 'Germany',
       'Operadora de Fondos GBM S.A. de C.V': 'Mexico',
       'Tongling Nonferrous Metals Group Co., Ltd., ESOP': 'China',
       'DSP BlackRock Invts Magrs Pvt. Ltd': 'India',
       'Lianchu Securities Co.Ltd.': 'China',
       'Mitsui Mining And Smelting Company Ltd, ESOP': 'Japan',
       'MS&AD Insurance Group Holdings Inc.': 'Japan'
       }

def fill_missing_countries(row, missing_countries):
    if pd.isna(row['Country']):
        return missing_countries.get(row['Financial_name'], row['Country'])
    return row['Country']




def make_foreign_influence(mineral, data):
  test_lo = data.query('mineral==mineral')
  test_pen = test_lo.groupby(['OWNER_COUNTRY','COUNTRY_NAME'])['value'].sum().reset_index()
  test_pen['Share'] = test_pen['value'] / test_pen.groupby('COUNTRY_NAME')['value'].transform('sum')
  test_pen['National'] = test_pen['COUNTRY_NAME']==test_pen['OWNER_COUNTRY']
  test_pen['National'] = test_pen['National'].astype(int)
  test_pen['Share_foreign'] = test_pen.groupby(['COUNTRY_NAME','National'])['Share'].transform('sum')
  test_pen = test_pen.groupby(['COUNTRY_NAME','National'])['Share_foreign'].first().reset_index()
  test_pen['National'] = test_pen['National'].map({0: 'Foreign control', 1: 'National control'})
  test_pen= test_pen[test_pen['Share_foreign'] != 0]
  return test_pen


def make_foreign_influence_soc(mineral, data, price):
  test_lo = data.query('mineral==mineral')
  test_lo = pd.merge(test_lo, price, how='left', on='mineral')
  test_lo['value_dollar'] = test_lo['value'] * test_lo['Price']
  test_pen = test_lo.groupby(['Country','COUNTRY_NAME'])['value_dollar'].sum().reset_index()
  test_pen['Share'] = test_pen['value_dollar'] / test_pen.groupby('COUNTRY_NAME')['value_dollar'].transform('sum')
  test_pen['National'] = test_pen['COUNTRY_NAME']==test_pen['Country']
  test_pen['National'] = test_pen['National'].astype(int)
  test_pen['Share_foreign'] = test_pen.groupby(['COUNTRY_NAME','National'])['Share'].transform('sum')
  test_pen = test_pen.groupby(['COUNTRY_NAME','National'])['Share_foreign'].first().reset_index()
  test_pen['National'] = test_pen['National'].map({0: 'Foreign control', 1: 'National control'})
  test_pen= test_pen[test_pen['Share_foreign'] != 0]
  return test_pen

def geopolitical_distance_alter(data, x_a, x_b, a_max):
    # x_a and x_b are arrays representing the values for country a and country b respectively
    # a_max is an array representing the maximum possible values for the variables
    
    # Ensure inputs are numpy arrays for element-wise operations
    #x_a = np.array(x_a)
    #x_b = np.array(x_b)
    #a_max = np.array(a_max)
    
    # Calculate the numerator of the S score
    numerator = np.sum((x_a - x_b) ** 2)

    max_dist_numb = len(data)
    
    # Calculate the denominator of the S score
    denominator = np.sum(a_max ** 2) * max_dist_numb * 0.5
    
    # Calculate the S score
    S_score = 1 - (numerator / denominator)
    
    # Calculate the Geopolitical Distance
    GPD = (-1) * S_score
    
    return GPD